//---------------------------------------------------------------
// Example for Virtuino MQTT app: Create a programmable timer uses an ESP8266 or ESP32 Board
// Created by Ilias Lamprou at Jun/10/2018
// You can check on your device after a successful connection here: https://shiftr.io/try.
// Original MQTT example by Joël Gähwiler https://github.com/256dpi/arduino-mqtt
//---------------------------------------------------------------

//#include <ESP8266WiFi.h>  // ESP8266
#include <WiFi.h>       // for ESP32 enable this line and disable the previous
#include <MQTTClient.h>

//------ wifi settings
char ssid[] = "WIFI NETWORK"; //  Change this to your network SSID (name).
char pass[] = "PASSWORD";  // Change this your network password

//------ MQTT broker settings and topics
const char* broker = "broker.shiftr.io"; 
char mqttUserName[] = "virtuino";         
char mqttPass[] = "1234567890";               

const char* topic_schedule = "schedule"; 

#include <Arduino_JSON.h>    //recommended for Json parsing

WiFiClient net;
MQTTClient client;
unsigned long lastMillis = 0;

String scheduleHolder="";


byte theClockDay=1;    // RTC day of week  sunday=1 
byte theClockHour=0;   // RTC hour 
byte theClockMinute=0; // RTC minute
byte theClockSecond=0; // RTC second

unsigned long t;

//========================================= connect
void connect() {
  Serial.print("\nconnecting to wifi.");
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(5000);
  }
  //--- create a random client id
  char clientID[] ="ESP8266_0000000000";  // For random generation of client ID.
  for (int i = 8; i <18 ; i++) clientID[i]=  char(48+random(10));
    
  Serial.print("\nconnecting to broker...");
  while (!client.connect(clientID,mqttUserName,mqttPass)) {
    Serial.print(".");
    delay(5000);
  }
  Serial.println("\nconnected!");
  client.subscribe(topic_schedule);
  // client.unsubscribe("motor_command");
}

//========================================= messageReceived
void messageReceived(String &topic, String &payload) {
  Serial.println("incoming: " + topic + " - " + payload);
  if (topic==topic_schedule){
    scheduleHolder=payload;
    Serial.println(scheduleHolder);
   }
   //else  if (topic==

}

//========================================= setup
//=========================================
void setup() {
  Serial.begin(9600);
  WiFi.begin(ssid, pass); // Note: Local domain names (e.g. "Computer.local" on OSX) are not supported by Arduino. You need to set the IP address directly.
  client.begin(broker, net);
  pinMode(LED_BUILTIN,OUTPUT);
  scheduleHolder.reserve(1000);         // 1000 bytes are reserved for the schedule string
  client.onMessage(messageReceived);
  connect();
}
//========================================= loop
//=========================================
void loop() {
  client.loop();
  delay(10);  // <- fixes some issues with WiFi stability
  if (!client.connected()){
        digitalWrite(LED_BUILTIN,LOW);
        connect();
  }
  else digitalWrite(LED_BUILTIN,HIGH);

  //--- demo code to check the schedule
  //--- the hour is inceased by one every second
 if (millis()-t>1000){   // check the schedule every second
  t=millis();
  theClockHour++;
  if (theClockHour==24){
    theClockHour=0;
    theClockDay++;
    if (theClockDay==8) theClockDay=1;
  }
  boolean st=getscheduleState(theClockDay,theClockHour,theClockMinute,theClockSecond);
  Serial.print("Day:"+String(theClockDay)+"  hour:"+String(theClockHour));
  if (st) Serial.println("--- ON");
  else Serial.println("--- OFF");
  
 }
  
}
//============================================================ 
// The function getscheduleState gets as inputs the current time and day of week (sunday = 1)

boolean getscheduleState(byte clockDoW, byte clockHour, byte clockMinute, byte clockSecond){
  boolean returnedState=false;
  JSONVar myObject = JSON.parse(scheduleHolder);
  if (JSON.typeof(myObject) == "undefined"){
    Serial.println("Parsing input failed!");
    return returnedState;                     
  }
  if (myObject.hasOwnProperty("items")) {
    JSONVar items = myObject["items"];
    unsigned long clockTime=(clockHour*3600)+(clockMinute*60)+clockSecond;
    byte pDoW=0;
    unsigned long pT=0;
    byte lastDay=0;
    unsigned long lastTime=0;
    byte lastState=0;
    for (int i=0; i<items.length();i++){
      JSONVar obj = items[i];
      //Serial.println(obj);
      int hour = obj["H"];
      int minute = obj["M"];
      int second = obj["S"];
      int st = obj["ST"];
      unsigned long t=(hour*3600)+(minute*60)+second;
      String DoW_string = (const char*) obj["D"];
      int DoW=8;
      byte dayIndex=clockDoW-1;
      for (int m=dayIndex;m>=0;m--){
        if (m==dayIndex) {
         if ((DoW_string[m]-48==1) & (t<=clockTime)){    
          DoW=m+1;
          break;
         }
        } 
        else
        if (DoW_string[m]-48==1){    
          DoW=m+1;
          break;
        }
      }
        if (DoW<=clockDoW){
            if (((DoW==pDoW) & (t>pT))|| (DoW>pDoW)) {
              pDoW=DoW;
              pT=t;
              if (st==1) returnedState=true; else returnedState=false;
            }
        }
        //-- hold the lastState of the schedule
        if (pDoW==0){
           byte lDay=0;
           for (byte m=0;m<7;m++)
              if (DoW_string[m]-48==1) lDay=m+1;
           if (((lDay==lastDay) & (t>lastTime)) || (lDay>lastDay)) {
             lastDay=lDay;
             lastTime=t; 
             lastState=st; 
           }
        }
        
   }
   if (pDoW==0) {
    returnedState=lastState;
   }
  
  }
 return returnedState;
  
}


